@@ -1,4 +1,4 @@
-/* Copyright 2011-2014 Free Software Foundation, Inc.
+/* Copyright 2011-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -31,6 +31,6 @@ main (void)
     break_me (); /* Try a condition-specific breakpoint.  */
 
   break_me (); /* Finally, try a breakpoint with commands.  */
-  return 0;
+  return 0; /* Return line. */
 }
 
